@@ -1,54 +1,76 @@
 package GD::SecurityImage;
 use strict;
-use vars qw[@ISA $VERSION];
+use vars qw[@ISA $AUTOLOAD $VERSION $BACKEND];
 use GD::SecurityImage::Styles;
 
-@ISA     = qw(GD::SecurityImage::Styles);
-$VERSION = '1.4_03';
+$VERSION = '1.57_01';
 
 sub import {
+   my $class   = shift;
+   my %opt     = scalar(@_) % 2 ? () : (@_);
+   # init/reset globals
+   $BACKEND = ''; # name of the back-end
+   @ISA     = ();
    # load the drawing interface
-   my $class = shift;
-   my %opt   = scalar(@_) % 2 ? () : (@_);
-   if (exists $opt{use_magick} and $opt{use_magick}) {
+   if (exists $opt{use_magick} && $opt{use_magick}) {
       require GD::SecurityImage::Magick;
-      push @ISA, qw(GD::SecurityImage::Magick);
+      $BACKEND = 'Magick';
+   } elsif (exists $opt{backend} && $opt{backend}) {
+      my $be = 'GD::SecurityImage::'.$opt{backend};
+      eval "require $be";
+      die "Unable to locate the $class back-end $be: $@" if $@;
+      $BACKEND = $opt{backend} eq 'AC' ? 'GD' : $opt{backend};
    } else {
       require GD::SecurityImage::GD;
-      push @ISA, qw(GD::SecurityImage::GD);
+      $BACKEND = 'GD';
    }
+   push @ISA, 'GD::SecurityImage::' . $BACKEND;
+   push @ISA, qw(GD::SecurityImage::Styles); # load styles
 }
 
 sub new {
    my $class = shift;
+      $BACKEND || die "You didn't import $class!";
    my %opt   = scalar @_ % 2 ? () : (@_);
    my $self  = {
-      IS_MAGICK       => defined($Image::Magick::VERSION) ? 1 : 0,
+      IS_MAGICK       => $BACKEND eq 'Magick',
+      IS_GD           => $BACKEND eq 'GD',
+      IS_CORE         => $BACKEND eq 'GD' || $BACKEND eq 'Magick',
+      DISABLED        => {}, # list of methods that a backend (or some older version of backend) can't do
       MAGICK          => {}, # Image::Magick configuration options
       GDBOX_EMPTY     => 0,  # GD::SecurityImage::GD::insert_text() failed?
       _RANDOM_NUMBER_ => '', # random security code
+      _RND_LIST_      => [], # random codes list
       _RNDMAX_        => 6,  # maximum number of characters in a random string.
       _COLOR_         => {}, # text and line colors
       _CREATECALLED_  => 0,  # create() called? (check for particle())
+      _TEXT_LOCATION_ => {}, # see set_tl
    };
    bless $self, $class;
    my %options = (
-               width      => $opt{width}               || 80,
-               height     => $opt{height}              || 30,
-               ptsize     => $opt{ptsize}              || 20,
-               lines      => $opt{lines}               || 10,
-               rndmax     => $opt{rndmax}              || $self->{_RNDMAX_},
-               rnd_data   => $opt{rnd_data}            || [0..9],
-               font       => $opt{font}                || '',
-               gd_font    => $self->gdf($opt{gd_font}) || '',
-               bgcolor    => $opt{bgcolor}             || [255, 255, 255],
-               send_ctobg => $opt{send_ctobg}          || 0,
-               frame      => defined($opt{frame}) ? $opt{frame} : 1,
-               scramble   => $opt{scramble}            || 0,
-               angle      => $opt{angle}               || 0,
-               thickness  => $opt{thickness}           || 0,
-               _ANGLES_   => [], # angle list for scrambled images
+      width      => $opt{width}               || 80,
+      height     => $opt{height}              || 30,
+      ptsize     => $opt{ptsize}              || 20,
+      lines      => $opt{lines}               || 10,
+      rndmax     => $opt{rndmax}              || $self->{_RNDMAX_},
+      rnd_data   => $opt{rnd_data}            || [0..9],
+      font       => $opt{font}                || '',
+      gd_font    => $self->gdf($opt{gd_font}) || '',
+      bgcolor    => $opt{bgcolor}             || [255, 255, 255],
+      send_ctobg => $opt{send_ctobg}          || 0,
+      frame      => defined($opt{frame}) ? $opt{frame} : 1,
+      scramble   => $opt{scramble}            || 0,
+      angle      => $opt{angle}               || 0,
+      thickness  => $opt{thickness}           || 0,
+      _ANGLES_   => [], # angle list for scrambled images
    );
+   if($opt{text_location} && ref $opt{text_location} && ref $opt{text_location} eq 'HASH') {
+      $self->{_TEXT_LOCATION_} = { %{$opt{text_location}}, _place_ => 1 };
+   } else {
+      $self->{_TEXT_LOCATION_}{_place_} = 0;
+   }
+   $self->{_RNDMAX_} = $options{rndmax}; 
+
    $self->{$_} = $options{$_} foreach keys %options;
    if($self->{angle}) { # validate angle
       $self->{angle} = 360 + $self->{angle} if $self->{angle} < 0;
@@ -72,6 +94,36 @@ sub new {
    return $self;
 }
 
+sub backends {
+   my $self  = shift;
+   my $class = ref($self) || $self;
+   my(@list, @dir_list);
+   foreach my $inc (@INC) {
+      my $dir = "$inc/GD/SecurityImage";
+      next unless -d $dir;
+      opendir DIR, $dir or die "opendir($dir) failed: $!";
+      my @dir = readdir DIR;
+      closedir DIR;
+      push @dir_list, $dir;
+      foreach my $file (@dir) {
+         next if -d $file;
+         next if $file =~ m[^\.];
+         next if $file =~ m[^(Styles|AC)\.pm$];
+         $file =~ s[\.pm$][];
+         push @list, $file;
+      }
+   }
+   if (defined wantarray) {
+      return @list;
+   } else {
+      print "Available back-ends in $class v$VERSION are:\n\t"
+            .join("\n\t", @list)
+            ."\n\n"
+            ."Search directories:\n\t"
+            .join("\n\t", @dir_list);
+   }
+}
+
 sub gdf {
    my $self = shift;
    return if $self->{IS_MAGICK};
@@ -103,22 +155,62 @@ sub random {
    return $self if defined wantarray;
 }
 
+sub cconvert { # convert color codes
+   # GD           : return color index number
+   # Image::Magick: return hex color code
+   my $self   = shift;
+   my $data   = shift || die "Empty parameter passed to cconvert!";
+   unless($self->{IS_CORE}) {
+      return $self->backend_cconvert($data);
+   }
+   my $is_hex = $self->is_hex($data);
+   if($data && $self->{IS_MAGICK} && $is_hex) {
+      return $data; # data is a hex color code and Image::Magick has hex support
+   }
+   if(  $data              &&
+      ! $is_hex            &&
+      ! ref($data)         &&
+      $data !~ m{[^0-9]}   &&
+      $data >= 0
+      ) {
+      if ($self->{IS_MAGICK}) {
+         die "The number '$data' can not be transformed to a color code!";
+      } else {
+         # data is a GD color index number ...
+         # ... or it is any number! since there is no way to determine this. 
+         # GD object' s rgb() method returns 0,0,0 upon failure...
+         return $data;
+      }
+   }
+   my @rgb = $self->h2r($data);
+   if(@rgb and $self->{IS_MAGICK}) {
+      return $data;
+   } else {
+      $data = [@rgb] if @rgb;
+      # initialize if not valid
+      if(not $data || not ref $data || ref $data ne 'ARRAY' || $#{$data} != 2) {
+         $data = [0, 0, 0];
+      }
+      foreach my $i (0..$#{$data}) { # check for bad values
+         $data->[$i] = 0 if $data->[$i] > 255 or $data->[$i] < 0;
+      }
+   }
+   return $self->{IS_MAGICK} ? $self->r2h(@{$data}) # convert to hex
+                             : $self->{image}->colorAllocate(@{$data});
+}
+
 sub create {
    my $self   = shift;
    my $method = shift || 'normal';  # ttf or normal
    my $style  = shift || 'default'; # default or rect or box
-   my $col1   = shift; # text color
-   my $col2   = shift; # line/box color
-      $col1   = [ 0, 0, 0] if(not $col1 || not ref $col1 || ref $col1 ne 'ARRAY' || $#{$col1} != 2);
-      $col2   = [ 0, 0, 0] if(not $col2 || not ref $col2 || ref $col2 ne 'ARRAY' || $#{$col2} != 2);
-   my %color  = (
-        text  => $self->{IS_MAGICK} ? $col1 : $self->{image}->colorAllocate(@{$col1}),
-        lines => $self->{IS_MAGICK} ? $col2 : $self->{image}->colorAllocate(@{$col2}),
-   );
+   my $col1   = shift || [ 0, 0, 0]; # text color
+   my $col2   = shift || [ 0, 0, 0]; # line/box color
 
    $self->{send_ctobg} = 0 if $style eq 'box'; # disable for that style
-
-   $self->{_COLOR_} = \%color; # set the color hash
+   $self->{_COLOR_} = { # set the color hash
+        text  => $self->cconvert($col1),
+        lines => $self->cconvert($col2),
+   };
 
    # be a smart module and auto-disable ttf if we are under a prehistoric GD
    unless ($self->{IS_MAGICK}) {
@@ -165,6 +257,54 @@ sub particle {
 
 sub raw {shift->{image}} # raw image object
 
+sub change_random {
+   my $self = shift;
+   my $new  = shift;
+   return unless defined $new;
+   push @{ $self->{_RND_LIST_} }, $self->{_RANDOM_NUMBER_};
+   $self->{_RANDOM_NUMBER_} = $new;
+}
+
+sub set_tl { # set text location
+   # x      => 'left|right',  # text-X
+   # y      => 'up|low|down', # text-Y
+   # strip  => 1|0,           # add strip?
+   # gd     => 1|0,           # use default GD font?
+   # ptsize => 10,            # point size
+   # color  => '#000000',     # text color
+   # scolor => '#FFFFFF',     # strip color
+   # text   => 'blah',        # modifies random code
+   my $self = shift;
+   my %o    = scalar(@_) % 2 ? () : (@_);
+   return unless %o;
+
+   $self->{_TEXT_LOCATION_}->{_place_} = 1;
+   $self->change_random(delete $o{text}) if $o{text};
+   $self->{_COLOR_}{text} = $self->cconvert(delete $o{color}) if $o{color};
+   $o{scolor}             = $self->cconvert($o{scolor})       if $o{scolor};
+   $self->{ptsize}        = delete $o{ptsize} if $o{ptsize};
+
+   $self->{scramble}      = 0; # disable
+   $self->{angle}         = 0; # disable RT:14618
+
+   $self->{_TEXT_LOCATION_}->{$_} = $o{$_} foreach keys %o;
+   $self;
+}
+
+sub add_strip { # adds a strip to the background of the text
+   my $self = shift;
+   my($x, $y, $box_w, $box_h) = @_;
+   my $tl    = $self->{_TEXT_LOCATION_};
+   my $black = $self->cconvert($self->{_COLOR_}{text} ? $self->{_COLOR_}{text} : [0,0,0]);
+   my $white = $self->cconvert($tl->{scolor}          ? $tl->{scolor}          : [255,255,255]);
+   my $x2    = $tl->{x} eq 'left' ? $box_w : $self->{width};
+   my $y2    = $self->{height} - $box_h;
+   my $i     = $self->{IS_MAGICK} ? $self : $self->{image};
+   my $up    = $tl->{y} eq 'up';
+   $i->filledRectangle($up ? ($x-1, 0, $x2, $y+1) : ($x-1, $y2-1, $x2  , $self->{height}  ), $black);
+   $i->filledRectangle($up ? ($x  , 1, $x2-2, $y) : ($x  , $y2  , $x2-2, $self->{height}-2), $white);
+}
+
 #--------------------[ PRIVATE ]--------------------#
 
 sub r2h {
@@ -176,6 +316,32 @@ sub r2h {
       $color;
 }
 
+sub h2r {
+   # Convert Hex to RGB
+   my $self  = shift;
+   my $color = shift;
+   return if ref $color;
+   my @rgb   = $color =~ m[^#([a-f0-9]{2})([a-f0-9]{2})([a-f0-9]{2})$]i;
+   return @rgb ? map { hex $_ } @rgb : undef;
+}
+
+sub is_hex {
+   my $self = shift;
+   my $data = shift;
+   return $data =~ m[^#([a-f0-9]{2})([a-f0-9]{2})([a-f0-9]{2})$]i;
+}
+
+sub AUTOLOAD {
+   my $self = shift;
+   (my $name = $AUTOLOAD) =~ s,.*:,,;
+   if ($name eq 'gdbox_empty') { # fake method for GD compatibility. only GD has this
+      return 0;
+   }
+   die "Unknown ".ref($self)." method '$name'!";
+}
+
+sub DESTROY {}
+
 1;
 
 __END__
@@ -259,7 +425,7 @@ necessary sub modules.
 The (so called) I<"Security Images"> are so popular. Most internet 
 software use these in their registration screens to block robot programs
 (which may register tons of  fake member accounts). Security images are
-basicly, graphical CAPTCHAs (Completely Automated Public Turing Test to 
+basicaly, graphical CAPTCHAs (Completely Automated Public Turing Test to 
 Tell Computers and Humans Apart). This module gives you a basic interface 
 to create such an image. The final output is the actual graphic data, 
 the mime type of the graphic and the created random string.
@@ -267,6 +433,28 @@ the mime type of the graphic and the created random string.
 The module also has some I<"styles"> that are used to create the background 
 of the image.
 
+If you are an C<Authen::Captcha> user, see L<GD::SecurityImage::AC>
+for migration from C<Authen::Captcha> to C<GD::SecurityImage>.
+
+=head1 COLOR PARAMETERS
+
+Version 1.51 and later of this module is a little smarter than the 
+older versions. You can now use RGB and HEX values as the color 
+parameters:
+
+   $color  = '#80C0F0';     # HEX
+   $color2 = [15, 100, 75]; # RGB
+   $i->create($meth, $style, $color, $color2)
+
+   $i->create(ttf => 'box', '#80C0F0', '#0F644B')
+
+RGB values must be passed as an array reference including the three
+I<B<R>ed>, I<B<G>reen> and I<B<B>lue> values.
+
+Color conversion is transparent to the user. You can use hex values
+under both C<GD> and C<Image::Magick>. They' ll be automagically converted
+to RGB if you are under C<GD>.
+
 =head1 METHODS
 
 =head2 new
@@ -316,8 +504,7 @@ The names are case-insensitive; you can pass lower-cased parameters.
 
 =item bgcolor
 
-The background color of the image. Passed as an arrayref with three 
-elements (red, green, blue).
+The background color of the image.
 
 =item send_ctobg
 
@@ -333,16 +520,21 @@ option is enabled by default.
 =item scramble
 
 If set, the characters will be scrambled. If you enable this option,
-be sure to use a wider image, since the characters will be separeted 
+be sure to use a wider image, since the characters will be separated 
 with three spaces.
 
 =item angle
 
-Sets the angle for scrambled characters. Beware that, if you pass
+Sets the angle for scrambled/normal characters. Beware that, if you pass
 an C<angle> parameter, the characters in your random string will have
 a fixed angle. If you do not set an C<angle> parameter, the angle(s)
 will be random.
 
+When the scramble option is not enabled, this parameter still controls
+the angle of the text. But, since the text will be centered inside the 
+image, using this parameter without scramble option will require a 
+taller image. Clipping will occur with smaller height values.
+
 Unlike the GD interface, C<angle> is in C<degree>s and can take values 
 between C<0> and C<360>.
 
@@ -355,9 +547,6 @@ Default values are C<1> for GD and C<0.6> for Image:Magick.
 
 The length of the random string. Default value is C<6>.
 
-B<Not necessary and will not be used if you pass your own random>
-B<string.>
-
 =item rnd_data
 
 Default character set used to create the random string is C<0..9>.
@@ -416,7 +605,7 @@ Draws circles.
 
 =item B<ellipse>
 
-Draws ellipses.
+Draws ellipses. 
 
 =item B<ec>
 
@@ -428,11 +617,15 @@ and circles.
 Note: if you have a (very) old version of GD, you may not be able 
 to use some of the styles.
 
+You can use this code to get all available style names:
+
+   my @styles = grep {s/^style_//} keys %GD::SecurityImage::Styles::;
+
 The last two arguments (C<$text_color> and C<$line_color>) are the 
-colors used in the image (text and line color -- respectively) and 
-they are passed as a 3-element (red, green and blue) arrayref.
+colors used in the image (text and line color -- respectively):
 
    $image->create($method, $style, [0,0,0], [200,200,200]);
+   $image->create($method, $style, '#000000', '#c8c8c8');
 
 =head2 particle
 
@@ -460,11 +653,11 @@ The color of the particles are the same as the color of your text
 =head2 out
 
 This method finally returns the created image, the mime type of the 
-image and the random number generated. Older versions of GD only supports
+image and the random number(s) generated. Older versions of GD only support
 C<gif> type, while new versions support C<jpeg> and C<png> 
 (B<update>: beginning with v2.15, GD resumed gif support).
 
-The returned mime type is either C<gif> or C<jpeg> for C<GD> and 
+The returned mime type is C<png> or C<gif> or C<jpeg> for C<GD> and 
 C<gif> for C<Image::Magick> (if you do not C<force> some other format).
 
 C<out> method accepts arguments:
@@ -517,53 +710,110 @@ Compression and quality operations are disabled by default.
 Depending on your usage of the module; returns the raw C<GD::Image> 
 object:
 
-   my $i = $image->raw;
-   print $i->png;
+   my $gd = $image->raw;
+   print $gd->png;
 
 or the raw C<Image::Magick> object:
 
-   my $i = $image->raw;
-   $i->Write("gif:-");
+   my $magick = $image->raw;
+   $magick->Write("gif:-");
 
-Can be usefull, if you want to modify the graphic yourself, or want to
-use another output format like C<png>.
+Can be usefull, if you want to modify the graphic yourself. If you 
+want to get an I<image type> see the C<force> option in C<out>.
+
+=head1 UTILITY METHODS
+
+=head2 backends
+
+Returns a list of available GD::SecurityImage back-ends.
+
+   my @be = GD::SecurityImage->backends;
+
+or
+
+   my @be = $image->backends;
+
+If called in a void context, prints a verbose list of available 
+GD::SecurityImage back-ends:
+
+   Available back-ends in GD::SecurityImage v1.55 are:
+           GD
+           Magick
+   
+   Search directories:
+              /some/@INC/dir/containing/GDSI
+
+you can see the output with this command:
+
+   perl -MGD::SecurityImage -e 'GD::SecurityImage->backends'
+
+or under windows:
+
+   perl -MGD::SecurityImage -e "GD::SecurityImage->backends"
+
+=begin BACKEND_AUTHORS
+
+If you want to write a new back-end to GD::SecurityImage, you must define 
+this mandatory methods.
+
+   init			initializes your image object
+   out			defines output format and returns the image data
+   insert_text		inserts text to the image
+   setPixel		sets a pixel' s color defined by it's (x,y) values
+   line			draws a line
+   rectangle		draws a rectangle
+   filledRectangle	draws a filled rectangle
+   ellipse		draws an ellipse
+   arc			draws an arc
+   setThickness		sets the thickness of the lines when drawing something
+
+and
+
+   backend_cconvert	for HEX & RGB color handling
+
+See GD::SecurityImage::Magick for the first part of methods and see 
+cconvert() method in GD::SecurityImage to define such a method. Your 
+backend_cconvert() method must be capable of handling both HEX and RGB 
+values. The parametes passed to drawing methods (like line()) are 
+in GD format. See the L<GD> module for examples.
+
+You can then name your distro as 'GD::SecurityImage::X' and anyone can use 
+it like:
+
+   use GD::SecurityImage backend => 'X';
+
+=end BACKEND_AUTHORS
 
 =head1 EXAMPLES
 
-See the tests in the distribution.
+See the tests in the distribution. Also see the demo program 
+"eg/demo.pl" for an C<Apache::Session> implementation of C<GD::SecurityImage>.
+
+Download the distribution from a CPAN mirror near you, if you don't have 
+the files.
 
 =head1 ERROR HANDLING
 
-Currently, the module does not check the return values of C<GD>'s and
-C<Image::Magick>' s methods. So, if an error occurs, you may just get 
-an empty image instead of die()ing.
+C<die> is called in some methods if something fails. You may need to 
+C<eval> your code to catch exceptions.
 
-=head1 SEE ALSO
+=head1 TIPS
 
-L<GD>, L<Image::Magick>, L<ImagePwd>, L<Authen::Captcha>, 
+If you look at the demo program (not just look at it, try to run it)
+you'll see that the random code changes after every request (successful 
+or not). If you do not change the random code after a failed request and 
+display the random code inside HTML (like I<"Wrong! It must be E<lt>randomE<gt>">),
+then you are doing a logical mistake, since the user (or robot) can now 
+copy & paste the random code into your validator without looking at the 
+security image and will pass the test. Just don't do that. Random code 
+must change after every validation.
 
-C<ImageCode> Perl Module (commercial): 
-L<http://www.progland.com/ImageCode.html>.
-
-The CAPTCHA project: L<http://www.captcha.net/>.
-
-=head1 CAVEAT EMPTOR
-
-=over 4
-
-=item *
-
-Using the default library C<GD> is a better choice. Since it is faster 
-and does not use that much memory, while C<Image::Magick> is slower and 
-uses more memory.
-
-=item *
-
-The internal random code generator is used B<only> for demonstration 
-purposes for this module. It may not be I<effective>. You must supply 
-your own random code and use this module to display it.
-
-=back
+If you want to be a little more strict, you can also add a timeout key 
+to the session (this feature currently does not exits in the demo) and
+expire the related random code after the timeout. Since robots can call 
+the image generator directly (without requiring the HTML form), they can 
+examine the image for a while without changing it. A timeout implemetation
+may prevent this.
 
 =head1 BUGS
 
@@ -582,14 +832,14 @@ Please upgrade to ImageMagick 6.0.4 or any newer version, if your ImageMagick
 version is smaller than 6.0.4 and you want to use Image::Magick as the backend
 for GD::SecurityImage.
 
-=item GD bugs
+=item GD bug
 
 =over 4
 
 =item path bug
 
-libgd and GD.pm does not like relative paths and paths that have spaces
-in it. If you pass a font path that is not an B<exact path> or a path that
+libgd and GD.pm don't like relative paths and paths that have spaces
+in them. If you pass a font path that is not an B<exact path> or a path that
 have a space in it, you may get an empty image. 
 
 To check if the module failed to find the ttf font (when using C<GD>), a new 
@@ -600,23 +850,22 @@ method added: C<gdbox_empty()>. It must be called after C<create()>:
 
 C<gdbox_empty()> always returns false, if you are using C<Image::Magick>.
 
-B<Note>: New versions of GD does not have this bug. You can upgrade to 
-v2.16 or any newer version to fix this.
+=item GIF - Old libgd or libgd without GIF support enabled
 
-=item png compression
+If your GD has a C<gif> method, but you get empty images with C<gif()>
+method, you have to update your libgd or compile it with GIF enabled.
 
-Not a bug, but older versions does not have this.
+You can test if C<gif> is working from the command line:
 
-Support for compression level argument to png() added in v2.07. If
-your GD version is smaller than this, compress option to C<out()>
-will be silently ignored.
+   perl -MGD -e '$_=GD::Image->new;$_->colorAllocate(0,0,0);print$_->gif'
 
-=item setThickness
+or under windows:
 
-Not a bug, but older versions does not have this.
+   perl -MGD -e "$_=GD::Image->new;$_->colorAllocate(0,0,0);print$_->gif"
 
-setThickness implemented in GD v2.07. If your GD version is smaller
-than that and you set thickness option, your code will probably C<die>.
+If it dies, your GD is very old. 
+If it prints nothing, your libgd was compiled without GIF enabled (upgrade or re-compile). 
+If it prints out a junk that starts with 'GIF87a', everything is OK.
 
 =back
 
@@ -624,17 +873,84 @@ than that and you set thickness option, your code will probably C<die>.
 
 Contact the author if you find any bugs. You can also send requests.
 
+=head1 CAVEAT EMPTOR
+
+=over 4
+
+=item *
+
+Using the default library C<GD> is a better choice. Since it is faster 
+and does not use that much memory, while C<Image::Magick> is slower and 
+uses more memory.
+
+=item *
+
+The internal random code generator is used B<only> for demonstration 
+purposes for this module. It may not be I<effective>. You must supply 
+your own random code and use this module to display it.
+
+=item [GD] png compression
+
+Support for compression level argument to png() added in v2.07. If
+your GD version is smaller than this, compress option to C<out()>
+will be silently ignored.
+
+=item [GD] setThickness
+
+setThickness implemented in GD v2.07. If your GD version is smaller
+than that and you set thickness option, nothing will happen.
+
+=item [GD] ellipse
+
+C<ellipse()> method added in GD 2.07. 
+
+If your GD version is smaller than 2.07 and you use C<ellipse>, 
+the C<default> style will be returned.
+
+If your GD is smaller than 2.07 and you use C<ec>, only the circles will
+be drawn.
+
+=back
+
+=head1 SEE ALSO
+
+=over 4
+
+=item *
+
+L<GD>, L<Image::Magick>, L<ImagePwd>, L<Authen::Captcha>.
+
+=item *
+
+L<GD::SecurityImage::AC>: C<Authen::Captcha> drop-in replacement module.
+
+=item *
+
+C<ImageCode> Perl Module (commercial): L<http://www.progland.com/ImageCode.html>.
+
+=item *
+
+The CAPTCHA project: L<http://www.captcha.net/>.
+
+=item *
+
+A definition of CAPTCHA (From Wikipedia, the free encyclopedia):
+L<http://en.wikipedia.org/wiki/Captcha>.
+
+=back
+
 =head1 AUTHOR
 
 Burak Grsoy, E<lt>burakE<64>cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2004 Burak Grsoy. All rights reserved.
+Copyright 2004-2005 Burak Grsoy. All rights reserved.
 
 =head1 LICENSE
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself. 
+This library is free software; you can redistribute it and/or modify 
+it under the same terms as Perl itself, either Perl version 5.8.6 or, 
+at your option, any later version of Perl 5 you may have available.
 
 =cut
