@@ -18,21 +18,12 @@ use constant MAX_COMPRESS => 100;
 
 use Image::Magick;
 
-$VERSION = "1.31";
-
-sub gdbox_empty {0} # fake method for GD compatibility.
-
-sub rgbx {
-   # Convert color data to hex for Image::Magick
-   my $self   = shift;
-   my @data   = (ref $_[0] and ref $_[0] eq 'ARRAY') ? (@{$_[0]}) : (@_);
-   return $self->r2h(@data);
-}
+$VERSION = '1.35';
 
 sub init {
    # Create the image object
    my $self = shift;
-   my $bg   = $self->rgbx($self->{bgcolor}); 
+   my $bg   = $self->cconvert($self->{bgcolor}); 
       $self->{image} = Image::Magick->new;
       $self->{image}->Set(size=> "$self->{width}x$self->{height}");
       $self->{image}->Read('null:' . $bg);
@@ -49,19 +40,17 @@ sub out {
       my %g = map {$_, 1} $self->{image}->QueryFormat;
       $type = $opt{force} if exists $g{$opt{force}};
    }
-   # compression JPEG  LosslessJPEG  Zip
    $self->{image}->Set(magick => $type);
    if ($opt{'compress'} and $type =~ m[^(png|jpeg)$]) {
-      #if ($type eq 'gif') {
-      #   $self->{image}->Set(compression => 'LZW');
-      #}
       if($type eq 'png') {
          $opt{'compress'} = MAX_COMPRESS;
          $self->{image}->Set(compression => 'Zip');
       }
       $self->{image}->Set(quality => $opt{'compress'});
    }
-   return $self->{image}->ImageToBlob, $type, $self->{_RANDOM_NUMBER_};
+   my @all_random = @{ $self->{_RND_LIST_} };
+      @all_random = ($self->{_RANDOM_NUMBER_}) unless scalar(@all_random) > 0;
+   return $self->{image}->ImageToBlob, $type, @all_random;
 }
 
 sub insert_text {
@@ -73,10 +62,13 @@ sub insert_text {
    my %same   = (font      => $self->{font},
                  encoding  => 'UTF-8',
                  pointsize => $self->{ptsize},
-                 fill      => $self->rgbx($self->{_COLOR_}{text}),
+                 fill      => $self->cconvert($self->{_COLOR_}{text}),
    );
    if ($self->{scramble}) {
       my $space = [$info->(' '), 0, ' ']; # get " " parameters
+      my @randomy;
+      my $sy = $space->[ASCENDER] || 1;
+      push(@randomy,  $_, - $_) foreach $sy/2, $sy/4, $sy/8;
       my @char;
       foreach (split //, $key) {
          push @char, [$info->($_), $self->random_angle, $_], $space, $space, $space;
@@ -86,24 +78,37 @@ sub insert_text {
       foreach my $magick (@char) {
          $total -= $magick->[WIDTH] * 2;
          $self->{image}->Annotate(text   => $magick->[CHAR],
-                                  x      => ($self->{width}  - $total - $magick->[WIDTH]   ) / 2,
-                                  y      => ($self->{height}          + $magick->[ASCENDER]) / 2,
+                                  x      =>  ($self->{width}  - $total - $magick->[WIDTH]   ) / 2,
+                                  y      => (($self->{height}          + $magick->[ASCENDER]) / 2) + $randomy[int rand @randomy],
                                   rotate => $magick->[ANGLE],
                                   %same);
       }
-      # stretch=>{Normal, UltraCondensed, ExtraCondensed, Condensed, SemiCondensed, SemiExpanded, Expanded, ExtraExpanded, UltraExpanded}
    } else {
       my @metric = $info->($key);
-      my $x = ($self->{width}  - $metric[WIDTH]   ) / 2;
-      my $y = ($self->{height} + $metric[ASCENDER]) / 2;
-      $self->{image}->Annotate(text => $key, x => $x, y => $y, %same);
+      my($x, $y);
+      my $tl = $self->{_TEXT_LOCATION_};
+      if ($tl->{_place_}) {
+         # put the text to one of the four corners in the image
+         $x = $tl->{x} eq 'left' ? 2                   : $self->{width}-$metric[WIDTH]-2;
+         $y = $tl->{y} eq 'up'   ? $metric[ASCENDER]+1 : $self->{height}-2;
+         $self->add_strip($x, $y, $metric[WIDTH], $metric[ASCENDER]) if $tl->{strip};
+      } else {
+         $x = ($self->{width}  - $metric[WIDTH]   ) / 2;
+         $y = ($self->{height} + $metric[ASCENDER]) / 2;
+      }
+      $self->{image}->Annotate(%same,
+         text   => $key, 
+         x      => $x, 
+         y      => $y,
+         rotate => $self->{angle} ? 360 - $self->{angle} : 0,
+      );
    }
 }
 
 sub setPixel {
    my $self = shift;
    my($x, $y, $color) = @_;
-   $self->{image}->Set("pixel[$x,$y]" => $self->rgbx($color) );
+   $self->{image}->Set("pixel[$x,$y]" => $self->cconvert($color) );
 }
 
 sub line {
@@ -112,7 +117,7 @@ sub line {
       $self->{image}->Draw(
          primitive   => "line",
          points      => "$x1,$y1 $x2,$y2",
-         stroke      => $self->rgbx($color),
+         stroke      => $self->cconvert($color),
          strokewidth => $self->{MAGICK}{strokewidth},
       );
 }
@@ -123,7 +128,7 @@ sub rectangle {
       $self->{image}->Draw(
          primitive   => "rectangle",
          points      => "$x1,$y1 $x2,$y2",
-         stroke      => $self->rgbx($color),
+         stroke      => $self->cconvert($color),
          strokewidth => $self->{MAGICK}{strokewidth},
       );
 }
@@ -134,8 +139,8 @@ sub filledRectangle {
       $self->{image}->Draw(
          primitive   => "rectangle",
          points      => "$x1,$y1 $x2,$y2",
-         fill        => $self->rgbx($color),
-         stroke      => $self->rgbx($color),
+         fill        => $self->cconvert($color),
+         stroke      => $self->cconvert($color),
          strokewidth => 0,
       );
 }
@@ -146,7 +151,7 @@ sub ellipse {
       $self->{image}->Draw(
          primitive   => "ellipse",
          points      => "$cx,$cy $width,$height 0,360",
-         stroke      => $self->rgbx($color),
+         stroke      => $self->cconvert($color),
          strokewidth => $self->{MAGICK}{strokewidth},
       );
 }
@@ -157,7 +162,7 @@ sub arc {
       $self->{image}->Draw(
          primitive   => "ellipse", # I couldn't do that with "arc" primitive. patches are welcome, but this seems to work :)
          points      => "$cx,$cy $width,$height $start,$end",
-         stroke      => $self->rgbx($color),
+         stroke      => $self->cconvert($color),
          strokewidth => $self->{MAGICK}{strokewidth},
       );
 }
@@ -196,11 +201,12 @@ Burak Grsoy, E<lt>burakE<64>cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2004 Burak Grsoy. All rights reserved.
+Copyright 2004-2005 Burak Grsoy. All rights reserved.
 
 =head1 LICENSE
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself. 
+This library is free software; you can redistribute it and/or modify 
+it under the same terms as Perl itself, either Perl version 5.8.6 or, 
+at your option, any later version of Perl 5 you may have available.
 
 =cut
